@@ -6,6 +6,7 @@
 @license: GNU GPL
 """
 import gtk
+gtk.gdk.threads_init()
 import hildon
 import os
 import time
@@ -189,15 +190,7 @@ class fMMS_GUI(hildon.Program):
 		self.apn = hildon.PickerButton(gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_HORIZONTAL)
 		self.apn.set_selector(self.selector)
 		self.apn.set_active(self.active_apn_index)
-		#self.apn.connect("value-changed", self.apn_picker_changed)
 
-
-		"""self.apn = hildon.Entry(gtk.HILDON_SIZE_FINGER_HEIGHT)
-		apn_nicename = self.config.get_apn_nicename()
-		if apn_nicename != None:
-			self.apn.set_text(self.config.get_apn_nicename())
-		else:
-			self.apn.set_text("Not defined")"""
 		apnHBox.pack_start(apn_label, False, True, 0)
 		apnHBox.pack_start(self.apn, True, True, 0)
 		
@@ -242,16 +235,9 @@ class fMMS_GUI(hildon.Program):
 		allVBox.pack_end(numberHBox, False, False, 0)
 		allVBox.pack_end(imgwidthHBox, False, False, 0)
 		
-		#save = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
-		#save.set_label("Save")
-		#save.connect('clicked', self.config_menu_button_clicked, apn)
-		
-		
-		#allVBox.pack_end(buttonHBox)
 		allVBox.show_all()
 		dialog.vbox.add(allVBox)
 		dialog.add_button("Save", gtk.RESPONSE_APPLY)
-		#dialog.show_all()
 		while 1:
 			ret = dialog.run()
 			ret2 = self.config_menu_button_clicked(ret)
@@ -260,13 +246,9 @@ class fMMS_GUI(hildon.Program):
 			
 		dialog.destroy()
 		return ret
-		
-	"""def apn_picker_changed(self, button, user_data=None):
-		current_selection = button.get_value()
-		print "Current selection : %s" % (current_selection)"""
 
+	""" selector for apn """
 	def create_apn_selector(self):
-		#Create a HildonTouchSelector with a single text column
 		selector = hildon.TouchSelector(text = True)
 		
 		apnlist = self.config.get_gprs_apns()
@@ -355,6 +337,11 @@ class fMMS_GUI(hildon.Program):
 	""" lets call it quits! """
 	def quit(self, *args):
 		gtk.main_quit()
+	
+	""" forces ui update, kinda... god this is AWESOME """
+	def force_ui_update(self):
+		while gtk.events_pending():
+			gtk.main_iteration(False)
 		
 	""" delete push message """
 	def delete_push(self, fname):
@@ -420,7 +407,9 @@ class fMMS_GUI(hildon.Program):
 		return
 	
 	""" action on redl contextmenu click """
+	# TODO: dont block on this?
 	def liststore_redl_clicked(self, widget):
+		hildon.hildon_gtk_window_set_progress_indicator(self.window, 1)
 		dialog = gtk.Dialog()
 		dialog.set_title("WARNING")
 		dialog.add_button(gtk.STOCK_YES, 1)
@@ -429,6 +418,9 @@ class fMMS_GUI(hildon.Program):
 		dialog.vbox.add(label)
 		dialog.show_all()
 		ret = dialog.run()
+		dialog.destroy()
+		self.force_ui_update()
+		
 		if ret == 1:
 			(model, miter) = self.treeview.get_selection().get_selected()
 			# the 4th value is the filename (start counting at 0)
@@ -441,7 +433,7 @@ class fMMS_GUI(hildon.Program):
 			except Exception, e:
 				print e, e.args
 				banner = hildon.hildon_banner_show_information(self.window, "", "fMMS: Operation failed")
-		dialog.destroy()
+			hildon.hildon_gtk_window_set_progress_indicator(self.window, 0)
 		return
 
 	""" long press on image creates this """
@@ -468,34 +460,24 @@ class fMMS_GUI(hildon.Program):
 
 	""" show the selected mms """		
 	def show_mms(self, treeview, path):
+		# Show loading indicator
+		hildon.hildon_gtk_window_set_progress_indicator(self.window, 1)
+		banner = hildon.hildon_banner_show_information(self.window, "", "fMMS: Opening message")
+		self.force_ui_update()
+		
 		print path
 		model = treeview.get_model()
 		miter = model.get_iter(path)
 		# the 4th value is the filename (start counting at 0)
 		filename = model.get_value(miter, 3)
 		
-		### ugly workaround to get it show instantly
-		bus = dbus.SystemBus()
-		proxy = bus.get_object('org.freedesktop.Notifications', '/org/freedesktop/Notifications')
-		interface = dbus.Interface(proxy,dbus_interface='org.freedesktop.Notifications')
-		interface.SystemNoteInfoprint ("fMMS: Opening message")
-		### end ugly workaround ###
-		
-		gobject.timeout_add(100, self.return_false)
-		hildon.hildon_gtk_window_set_progress_indicator(self.window, 1)
-		# this is ugly i know, but i really want it to work!
-		gobject.timeout_add(50, self.return_false)
 		try:
 			self.show_new_window(filename)
 		except:
 			pass
 		# this is ugly i know, but i really want it to work!
-		gobject.timeout_add(50, self.return_false)
 		hildon.hildon_gtk_window_set_progress_indicator(self.window, 0)
 		
-	def return_false(self):
-		return False
-		
 	""" open a new window containing mms specified by fname """		
 	def show_new_window(self, fname):
 		win = hildon.StackableWindow()
@@ -547,7 +529,6 @@ class fMMS_GUI(hildon.Program):
 		
 		allVBox = gtk.VBox()
 		
-		#print fname
 		fullpath = self._mmsdir + "/" + str(fname) + "/headers"
 		try: 
 			fp = open(fullpath, 'r')
@@ -563,14 +544,8 @@ class fMMS_GUI(hildon.Program):
 		allVBox.show_all()
 		
 		pan.add_with_viewport(allVBox)
-		#pan.add(allVBox)
-		#dialogVBox.add(pan)
-		#dialogVBox.show_all()
-		
-		#dialog.vbox.add(dialogVBox)
 		dialog.vbox.add(pan)
 		dialog.vbox.show_all()
-		#dialog.show_all()
 		ret = dialog.run()
 		
 		dialog.destroy()
@@ -589,6 +564,9 @@ class fMMS_GUI(hildon.Program):
 	    fetches the mms if its not downloaded         """
 	# TODO: better file detection/support
 	def _parse_mms(self, filename, container):
+		hildon.hildon_gtk_window_set_progress_indicator(self.window, 1)
+		self.force_ui_update()
+		
 		if not self._is_fetched_push(filename):
 			self._get_mms_from_push(self._pushdir + filename)
 				
@@ -641,6 +619,8 @@ class fMMS_GUI(hildon.Program):
 				
 		textview.set_buffer(textbuffer)
 		container.add(textview)
+		hildon.hildon_gtk_window_set_progress_indicator(self.window, 0)
+		
 		
 	""" action on click on image/button """
 	def mms_img_clicked(self, widget, data):
@@ -695,8 +675,8 @@ class fMMS_GUI(hildon.Program):
 	""" this function requires the fname to be the fullpath """
 	# TODO: dont require fullpath
 	def _get_mms_from_push(self, fname):
+		
 		plist = self._read_push_as_list(fname)
-
 		try:
 			sndr = plist['From']
 		except:
@@ -710,10 +690,8 @@ class fMMS_GUI(hildon.Program):
 		
 		### ugly hack as well
 		# TODO: clean up
-		bus = dbus.SystemBus()
-		proxy = bus.get_object('org.freedesktop.Notifications', '/org/freedesktop/Notifications')
-		interface = dbus.Interface(proxy,dbus_interface='org.freedesktop.Notifications')
-		interface.SystemNoteInfoprint ("fMMS: Attempting to download MMS from server.")
+		hildon.hildon_banner_show_information(self.window, "", "fMMS: Attempting to download MMS from server.")
+		self.force_ui_update()
 		
 		path = PUSH._get_mms_message(url, trans_id)
 		PUSH.decodeMMS(path)
@@ -724,8 +702,12 @@ class fMMS_GUI(hildon.Program):
 
 	def run(self):
 		self.window.show_all()
+		gtk.gdk.threads_enter()
 		gtk.main()
+		gtk.gdk.threads_leave()
 		
 if __name__ == "__main__":
 	app = fMMS_GUI()
-	app.run()
\ No newline at end of file
+	app.run()
+	
+	
\ No newline at end of file
