@@ -8,7 +8,7 @@
 # The docstrings in this module contain epytext markup; API documentation
 # may be created by processing this file with epydoc: http://epydoc.sf.net
 """
-Library for WAP transport, original by Francois Aucamp
+Library for MMS message creation, original by Francois Aucamp
 Modified by Nick Leppnen Larsson for use in Maemo5/Fremantle on the Nokia N900.
 
 @author: Francois Aucamp <faucamp@csir.co.za>
@@ -28,15 +28,19 @@ class MMSMessage:
     
     @note: References used in this class: [1][2][3][4][5]
     """
-    def __init__(self):
+    def __init__(self, noHeaders=None):
+    	self.noHeaders = noHeaders
         transactionid = random.randint(0,100000)
         self._pages = []
         self._dataParts = []
         self._metaTags = {}
-        self.headers = {'Message-Type' : 'm-send-req',
-                        'Transaction-Id' : transactionid,
-                        'MMS-Version' : '1.0',
-                        'Content-Type' : ('application/vnd.wap.multipart.mixed', {'Start': '<smil>', 'Type': 'application/smil'})}
+        if self.noHeaders == None:
+		self.headers = {'Message-Type' : 'm-send-req',
+				'Transaction-Id' : transactionid,
+				'MMS-Version' : '1.0',
+				'Content-Type' : ('application/vnd.wap.multipart.mixed', {'Start': '<smil>', 'Type': 'application/smil'})}
+	else:
+		self.headers = {}
         self.width = 320
         self.height = 240
         self.transactionID = transactionid
@@ -201,7 +205,7 @@ class MMSMessage:
         @rtype array.array('B')
         """
         import mms_pdu
-        encoder = mms_pdu.MMSEncoder()
+        encoder = mms_pdu.MMSEncoder(self.noHeaders)
         return encoder.encode(self)
 
 
